@@ -31,7 +31,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "cogl-config.h"
 #endif
 
 #include "cogl-i18n-private.h"
