@@ -35,6 +35,8 @@
 #include "meta-cursor-renderer.h"
 #include "meta-monitor-manager-private.h"
 #include "backends/meta-pointer-constraint.h"
+#include "backends/meta-renderer.h"
+#include "core/util-private.h"
 
 #define DEFAULT_XKB_RULES_FILE "evdev"
 #define DEFAULT_XKB_MODEL "pc105+inet"
@@ -60,12 +62,15 @@ struct _MetaBackendClass
 {
   GObjectClass parent_class;
 
+  ClutterBackend * (* create_clutter_backend) (MetaBackend *backend);
+
   void (* post_init) (MetaBackend *backend);
 
   MetaIdleMonitor * (* create_idle_monitor) (MetaBackend *backend,
                                              int          device_id);
   MetaMonitorManager * (* create_monitor_manager) (MetaBackend *backend);
   MetaCursorRenderer * (* create_cursor_renderer) (MetaBackend *backend);
+  MetaRenderer * (* create_renderer) (MetaBackend *backend);
 
   gboolean (* grab_device) (MetaBackend *backend,
                             int          device_id,
@@ -99,10 +104,15 @@ struct _MetaBackendClass
                                            double       *dy_unaccel);
 };
 
+void meta_init_backend (MetaBackendType backend_type);
+
+ClutterBackend * meta_backend_get_clutter_backend (MetaBackend *backend);
+
 MetaIdleMonitor * meta_backend_get_idle_monitor (MetaBackend *backend,
                                                  int          device_id);
 MetaMonitorManager * meta_backend_get_monitor_manager (MetaBackend *backend);
 MetaCursorRenderer * meta_backend_get_cursor_renderer (MetaBackend *backend);
+MetaRenderer * meta_backend_get_renderer (MetaBackend *backend);
 
 gboolean meta_backend_grab_device (MetaBackend *backend,
                                    int          device_id,
@@ -130,4 +140,10 @@ gboolean meta_backend_get_relative_motion_deltas (MetaBackend *backend,
 void meta_backend_set_client_pointer_constraint (MetaBackend *backend,
                                                  MetaPointerConstraint *constraint);
 
+ClutterBackend * meta_backend_get_clutter_backend (MetaBackend *backend);
+
+void meta_backend_monitors_changed (MetaBackend *backend);
+
+gboolean meta_is_stage_views_enabled (void);
+
 #endif /* META_BACKEND_PRIVATE_H */
