@@ -26,6 +26,7 @@
 
 #include <stdlib.h>
 
+#include <clutter/clutter-mutter.h>
 #include <meta/meta-backend.h>
 #include "meta-backend-private.h"
 #include "meta-input-settings-private.h"
@@ -62,7 +63,9 @@ struct _MetaBackendPrivate
   MetaMonitorManager *monitor_manager;
   MetaCursorRenderer *cursor_renderer;
   MetaInputSettings *input_settings;
+  MetaRenderer *renderer;
 
+  ClutterBackend *clutter_backend;
   ClutterActor *stage;
 
   guint device_update_idle_id;
@@ -113,11 +116,11 @@ center_pointer (MetaBackend *backend)
                              primary->rect.y + primary->rect.height / 2);
 }
 
-static void
-on_monitors_changed (MetaMonitorManager *monitors,
-                     gpointer user_data)
+void
+meta_backend_monitors_changed (MetaBackend *backend)
 {
-  MetaBackend *backend = META_BACKEND (user_data);
+  MetaMonitorManager *monitor_manager =
+    meta_backend_get_monitor_manager (backend);
   ClutterDeviceManager *manager = clutter_device_manager_get_default ();
   ClutterInputDevice *device = clutter_device_manager_get_core_device (manager, CLUTTER_POINTER_DEVICE);
   ClutterPoint point;
@@ -127,7 +130,8 @@ on_monitors_changed (MetaMonitorManager *monitors,
   if (clutter_input_device_get_coords (device, NULL, &point))
     {
       /* If we're outside all monitors, warp the pointer back inside */
-      if (meta_monitor_manager_get_monitor_at_point (monitors, point.x, point.y) < 0)
+      if (meta_monitor_manager_get_monitor_at_point (monitor_manager,
+                                                     point.x, point.y) < 0)
         center_pointer (backend);
     }
 }
@@ -275,8 +279,6 @@ meta_backend_real_post_init (MetaBackend *backend)
 
   priv->monitor_manager = create_monitor_manager (backend);
 
-  g_signal_connect (priv->monitor_manager, "monitors-changed",
-                    G_CALLBACK (on_monitors_changed), backend);
   meta_backend_sync_screen_size (backend);
 
   priv->cursor_renderer = META_BACKEND_GET_CLASS (backend)->create_cursor_renderer (backend);
@@ -343,15 +345,6 @@ meta_backend_real_ungrab_device (MetaBackend *backend,
   return TRUE;
 }
 
-static void
-meta_backend_real_update_screen_size (MetaBackend *backend,
-                                      int width, int height)
-{
-  MetaBackendPrivate *priv = meta_backend_get_instance_private (backend);
-
-  clutter_actor_set_size (priv->stage, width, height);
-}
-
 static void
 meta_backend_real_select_stage_events (MetaBackend *backend)
 {
@@ -369,18 +362,27 @@ meta_backend_real_get_relative_motion_deltas (MetaBackend *backend,
   return FALSE;
 }
 
+static void
+meta_backend_constructed (GObject *object)
+{
+  MetaBackend *backend = META_BACKEND (object);
+  MetaBackendPrivate *priv = meta_backend_get_instance_private (backend);
+
+  priv->renderer = META_BACKEND_GET_CLASS (backend)->create_renderer (backend);
+}
+
 static void
 meta_backend_class_init (MetaBackendClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->constructed = meta_backend_constructed;
   object_class->finalize = meta_backend_finalize;
 
   klass->post_init = meta_backend_real_post_init;
   klass->create_cursor_renderer = meta_backend_real_create_cursor_renderer;
   klass->grab_device = meta_backend_real_grab_device;
   klass->ungrab_device = meta_backend_real_ungrab_device;
-  klass->update_screen_size = meta_backend_real_update_screen_size;
   klass->select_stage_events = meta_backend_real_select_stage_events;
   klass->get_relative_motion_deltas = meta_backend_real_get_relative_motion_deltas;
 
@@ -448,6 +450,16 @@ meta_backend_get_cursor_renderer (MetaBackend *backend)
   return priv->cursor_renderer;
 }
 
+/**
+ * meta_backend_get_renderer: (skip)
+ */
+MetaRenderer * meta_backend_get_renderer (MetaBackend *backend)
+{
+  MetaBackendPrivate *priv = meta_backend_get_instance_private (backend);
+
+  return priv->renderer;
+}
+
 /**
  * meta_backend_grab_device: (skip)
  */
@@ -610,30 +622,6 @@ meta_backend_set_client_pointer_constraint (MetaBackend           *backend,
     backend->client_pointer_constraint = g_object_ref (constraint);
 }
 
-static GType
-get_backend_type (void)
-{
-#if defined(CLUTTER_WINDOWING_X11)
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_X11))
-    return META_TYPE_BACKEND_X11;
-#endif
-
-#if defined(CLUTTER_WINDOWING_EGL) && defined(HAVE_NATIVE_BACKEND)
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_EGL))
-    return META_TYPE_BACKEND_NATIVE;
-#endif
-
-  g_assert_not_reached ();
-}
-
-static void
-meta_create_backend (void)
-{
-  /* meta_backend_init() above install the backend globally so
-   * so meta_get_backend() works even during initialization. */
-  g_object_new (get_backend_type (), NULL);
-}
-
 /* Mutter is responsible for pulling events off the X queue, so Clutter
  * doesn't need (and shouldn't) run its normal event source which polls
  * the X fd, but we do have to deal with dispatching events that accumulate
@@ -682,6 +670,54 @@ static GSourceFuncs event_funcs = {
   event_dispatch
 };
 
+ClutterBackend *
+meta_backend_get_clutter_backend (MetaBackend *backend)
+{
+  MetaBackendPrivate *priv = meta_backend_get_instance_private (backend);
+
+  if (!priv->clutter_backend)
+    {
+      priv->clutter_backend =
+        META_BACKEND_GET_CLASS (backend)->create_clutter_backend (backend);
+    }
+
+  return priv->clutter_backend;
+}
+
+static ClutterBackend *
+meta_get_clutter_backend (void)
+{
+  MetaBackend *backend = meta_get_backend ();
+
+  return meta_backend_get_clutter_backend (backend);
+}
+
+void
+meta_init_backend (MetaBackendType backend_type)
+{
+  GType type;
+
+  switch (backend_type)
+    {
+    case META_BACKEND_TYPE_X11:
+      type = META_TYPE_BACKEND_X11;
+      break;
+
+#ifdef HAVE_NATIVE_BACKEND
+    case META_BACKEND_TYPE_NATIVE:
+      type = META_TYPE_BACKEND_NATIVE;
+      break;
+#endif
+
+    default:
+      g_assert_not_reached ();
+    }
+
+  /* meta_backend_init() above install the backend globally so
+   * so meta_get_backend() works even during initialization. */
+  g_object_new (type, NULL);
+}
+
 /**
  * meta_clutter_init: (skip)
  */
@@ -691,7 +727,7 @@ meta_clutter_init (void)
   ClutterSettings *clutter_settings;
   GSource *source;
 
-  meta_create_backend ();
+  clutter_set_custom_backend_func (meta_get_clutter_backend);
 
   if (clutter_init (NULL, NULL) != CLUTTER_INIT_SUCCESS)
     {
@@ -712,3 +748,19 @@ meta_clutter_init (void)
 
   meta_backend_post_init (_backend);
 }
+
+gboolean
+meta_is_stage_views_enabled (void)
+{
+  const gchar *mutter_stage_views;
+
+  if (!meta_is_wayland_compositor ())
+    return FALSE;
+
+  mutter_stage_views = g_getenv ("MUTTER_STAGE_VIEWS");
+
+  if (!mutter_stage_views)
+    return FALSE;
+
+  return strcmp (mutter_stage_views, "1") == 0;
+}
