@@ -25,11 +25,14 @@
 
 #include "meta-monitor-manager-kms.h"
 #include "meta-monitor-config.h"
+#include "meta-backend-private.h"
+#include "meta-renderer-native.h"
 
 #include <string.h>
 #include <stdlib.h>
 #include <clutter/clutter.h>
 
+#include <drm.h>
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
@@ -73,11 +76,20 @@ typedef struct {
   uint32_t rotation_map[ALL_TRANSFORMS];
 } MetaCRTCKms;
 
+typedef struct
+{
+  GSource source;
+
+  gpointer fd_tag;
+  MetaMonitorManagerKms *manager_kms;
+} MetaKmsSource;
+
 struct _MetaMonitorManagerKms
 {
   MetaMonitorManager parent_instance;
 
   int fd;
+  MetaKmsSource *source;
 
   drmModeConnector **connectors;
   unsigned int       n_connectors;
@@ -85,6 +97,8 @@ struct _MetaMonitorManagerKms
   GUdevClient *udev;
 
   GSettings *desktop_settings;
+
+  gboolean page_flips_not_supported;
 };
 
 struct _MetaMonitorManagerKmsClass
@@ -952,9 +966,6 @@ meta_monitor_manager_kms_set_power_save_mode (MetaMonitorManager *manager,
                                               MetaPowerSave       mode)
 {
   MetaMonitorManagerKms *manager_kms = META_MONITOR_MANAGER_KMS (manager);
-  ClutterBackend *backend;
-  CoglContext *cogl_context;
-  CoglDisplay *cogl_display;
   uint64_t state;
   unsigned i;
 
@@ -994,21 +1005,6 @@ meta_monitor_manager_kms_set_power_save_mode (MetaMonitorManager *manager,
                           meta_output->name, strerror (errno));
         }
     }
-
-  backend = clutter_get_default_backend ();
-  cogl_context = clutter_backend_get_cogl_context (backend);
-  cogl_display = cogl_context_get_display (cogl_context);
-
-  for (i = 0; i < manager->n_crtcs; i++)
-    cogl_kms_display_set_ignore_crtc (cogl_display, manager->crtcs[i].crtc_id,
-                                      mode != META_POWER_SAVE_ON);
-}
-
-static void
-crtc_free (CoglKmsCrtc *crtc)
-{
-  g_free (crtc->connectors);
-  g_slice_free (CoglKmsCrtc, crtc);
 }
 
 static void
@@ -1061,39 +1057,20 @@ meta_monitor_manager_kms_apply_configuration (MetaMonitorManager *manager,
                                               unsigned int         n_outputs)
 {
   MetaMonitorManagerKms *manager_kms = META_MONITOR_MANAGER_KMS (manager);
-  ClutterBackend *backend;
-  CoglContext *cogl_context;
-  CoglDisplay *cogl_display;
   unsigned i;
-  GPtrArray *cogl_crtcs;
   int screen_width, screen_height;
-  gboolean ok;
-  GError *error;
 
-  cogl_crtcs = g_ptr_array_new_full (manager->n_crtcs, (GDestroyNotify)crtc_free);
   screen_width = 0; screen_height = 0;
   for (i = 0; i < n_crtcs; i++)
     {
       MetaCRTCInfo *crtc_info = crtcs[i];
       MetaCRTC *crtc = crtc_info->crtc;
       MetaCRTCKms *crtc_kms = crtc->driver_private;
-      CoglKmsCrtc *cogl_crtc;
 
       crtc->is_dirty = TRUE;
 
-      cogl_crtc = g_slice_new0 (CoglKmsCrtc);
-      g_ptr_array_add (cogl_crtcs, cogl_crtc);
-
       if (crtc_info->mode == NULL)
         {
-          cogl_crtc->id = crtc->crtc_id;
-          cogl_crtc->x = 0;
-          cogl_crtc->y = 0;
-          cogl_crtc->count = 0;
-          memset (&cogl_crtc->mode, 0, sizeof (drmModeModeInfo));
-          cogl_crtc->connectors = NULL;
-          cogl_crtc->count = 0;
-
           crtc->rect.x = 0;
           crtc->rect.y = 0;
           crtc->rect.width = 0;
@@ -1103,31 +1080,19 @@ meta_monitor_manager_kms_apply_configuration (MetaMonitorManager *manager,
       else
         {
           MetaMonitorMode *mode;
-          uint32_t *connectors;
-          unsigned int j, n_connectors;
+          unsigned int j;
           int width, height;
 
           mode = crtc_info->mode;
 
-          cogl_crtc->id = crtc->crtc_id;
-          cogl_crtc->x = crtc_info->x;
-          cogl_crtc->y = crtc_info->y;
-          cogl_crtc->count = n_connectors = crtc_info->outputs->len;
-          cogl_crtc->connectors = connectors = g_new (uint32_t, n_connectors);
-
-          for (j = 0; j < n_connectors; j++)
+          for (j = 0; j < crtc_info->outputs->len; j++)
             {
               MetaOutput *output = g_ptr_array_index (crtc_info->outputs, j);
 
-              connectors[j] = output->winsys_id;
-
               output->is_dirty = TRUE;
               output->crtc = crtc;
             }
 
-          memcpy (&cogl_crtc->mode, crtc_info->mode->driver_private,
-                  sizeof (drmModeModeInfo));
-
           if (meta_monitor_transform_is_rotated (crtc_info->transform))
             {
               width = mode->height;
@@ -1163,7 +1128,6 @@ meta_monitor_manager_kms_apply_configuration (MetaMonitorManager *manager,
   for (i = 0; i < manager->n_crtcs; i++)
     {
       MetaCRTC *crtc = &manager->crtcs[i];
-      CoglKmsCrtc *cogl_crtc;
 
       crtc->logical_monitor = NULL;
 
@@ -1173,17 +1137,6 @@ meta_monitor_manager_kms_apply_configuration (MetaMonitorManager *manager,
           continue;
         }
 
-      cogl_crtc = g_slice_new0 (CoglKmsCrtc);
-      g_ptr_array_add (cogl_crtcs, cogl_crtc);
-
-      cogl_crtc->id = crtc->crtc_id;
-      cogl_crtc->x = 0;
-      cogl_crtc->y = 0;
-      cogl_crtc->count = 0;
-      memset (&cogl_crtc->mode, 0, sizeof (drmModeModeInfo));
-      cogl_crtc->connectors = NULL;
-      cogl_crtc->count = 0;
-
       crtc->rect.x = 0;
       crtc->rect.y = 0;
       crtc->rect.width = 0;
@@ -1191,22 +1144,6 @@ meta_monitor_manager_kms_apply_configuration (MetaMonitorManager *manager,
       crtc->current_mode = NULL;
     }
 
-  backend = clutter_get_default_backend ();
-  cogl_context = clutter_backend_get_cogl_context (backend);
-  cogl_display = cogl_context_get_display (cogl_context);
-
-  error = NULL;
-  ok = cogl_kms_display_set_layout (cogl_display, screen_width, screen_height,
-                                    (CoglKmsCrtc**)cogl_crtcs->pdata, cogl_crtcs->len, &error);
-  g_ptr_array_unref (cogl_crtcs);
-
-  if (!ok)
-    {
-      meta_warning ("Applying display configuration failed: %s\n", error->message);
-      g_error_free (error);
-      return;
-    }
-
   for (i = 0; i < n_outputs; i++)
     {
       MetaOutputInfo *output_info = outputs[i];
@@ -1293,20 +1230,41 @@ on_uevent (GUdevClient *client,
   meta_monitor_manager_on_hotplug (manager);
 }
 
+static gboolean
+kms_event_check (GSource *source)
+{
+  MetaKmsSource *kms_source = (MetaKmsSource *) source;
+
+  return g_source_query_unix_fd (source, kms_source->fd_tag) & G_IO_IN;
+}
+
+static gboolean
+kms_event_dispatch (GSource    *source,
+                    GSourceFunc callback,
+                    gpointer    user_data)
+{
+  MetaKmsSource *kms_source = (MetaKmsSource *) source;
+
+  meta_monitor_manager_kms_wait_for_flip (kms_source->manager_kms);
+
+  return G_SOURCE_CONTINUE;
+}
+
+static GSourceFuncs kms_event_funcs = {
+  NULL,
+  kms_event_check,
+  kms_event_dispatch
+};
+
 static void
 meta_monitor_manager_kms_init (MetaMonitorManagerKms *manager_kms)
 {
-  ClutterBackend *backend;
-  CoglContext *cogl_context;
-  CoglDisplay *cogl_display;
-  CoglRenderer *cogl_renderer;
+  MetaBackend *backend = meta_get_backend ();
+  MetaRenderer *renderer = meta_backend_get_renderer (backend);
+  MetaRendererNative *renderer_native = META_RENDERER_NATIVE (renderer);
+  GSource *source;
 
-  backend = clutter_get_default_backend ();
-  cogl_context = clutter_backend_get_cogl_context (backend);
-  cogl_display = cogl_context_get_display (cogl_context);
-  cogl_renderer = cogl_display_get_renderer (cogl_display);
-
-  manager_kms->fd = cogl_kms_renderer_get_kms_fd (cogl_renderer);
+  manager_kms->fd = meta_renderer_native_get_kms_fd (renderer_native);
 
   drmSetClientCap (manager_kms->fd, DRM_CLIENT_CAP_UNIVERSAL_PLANES, 1);
 
@@ -1315,9 +1273,192 @@ meta_monitor_manager_kms_init (MetaMonitorManagerKms *manager_kms)
   g_signal_connect (manager_kms->udev, "uevent",
                     G_CALLBACK (on_uevent), manager_kms);
 
+  source = g_source_new (&kms_event_funcs, sizeof (MetaKmsSource));
+  manager_kms->source = (MetaKmsSource *) source;
+  manager_kms->source->fd_tag = g_source_add_unix_fd (source,
+                                                      manager_kms->fd,
+                                                      G_IO_IN | G_IO_ERR);
+  manager_kms->source->manager_kms = manager_kms;
+  g_source_attach (source, NULL);
+
   manager_kms->desktop_settings = g_settings_new ("org.gnome.desktop.interface");
 }
 
+static void
+get_crtc_connectors (MetaMonitorManager *manager,
+                     MetaCRTC           *crtc,
+                     uint32_t          **connectors,
+                     unsigned int       *n_connectors)
+{
+  GArray *connectors_array = NULL;
+  unsigned int i;
+
+  for (i = 0; i < manager->n_outputs; i++)
+    {
+      MetaOutput *output = &manager->outputs[i];
+
+      if (output->crtc == crtc)
+        {
+          if (!connectors_array)
+            connectors_array = g_array_new (FALSE, FALSE, sizeof (uint32_t));
+          g_array_append_val (connectors_array, output->winsys_id);
+        }
+    }
+
+  if (connectors_array)
+    {
+      *connectors = (uint32_t *) connectors_array->data;
+      *n_connectors = connectors_array->len;
+    }
+  else
+    {
+      *connectors = NULL;
+      *n_connectors = 0;
+    }
+}
+
+void
+meta_monitor_manager_kms_apply_crtc_mode (MetaMonitorManagerKms *manager_kms,
+                                          MetaCRTC              *crtc,
+                                          int                    x,
+                                          int                    y,
+                                          uint32_t               fb_id)
+{
+  MetaMonitorManager *manager = META_MONITOR_MANAGER (manager_kms);
+  uint32_t *connectors;
+  unsigned int n_connectors;
+  drmModeModeInfo *mode;
+
+  get_crtc_connectors (manager, crtc, &connectors, &n_connectors);
+
+  if (connectors)
+    mode = crtc->current_mode->driver_private;
+  else
+    mode = NULL;
+
+  if (drmModeSetCrtc (manager_kms->fd,
+                      crtc->crtc_id,
+                      fb_id,
+                      x, y,
+                      connectors, n_connectors,
+                      mode) != 0)
+    g_warning ("Failed to set CRTC mode %s: %m", crtc->current_mode->name);
+
+  g_free (connectors);
+}
+
+static void
+invoke_flip_closure (GClosure *flip_closure)
+{
+  GValue param = G_VALUE_INIT;
+
+  g_value_init (&param, G_TYPE_POINTER);
+  g_value_set_pointer (&param, flip_closure);
+  g_closure_invoke (flip_closure, NULL, 1, &param, NULL);
+  g_closure_unref (flip_closure);
+}
+
+gboolean
+meta_monitor_manager_kms_is_crtc_active (MetaMonitorManagerKms *manager_kms,
+                                         MetaCRTC              *crtc)
+{
+  MetaMonitorManager *manager = META_MONITOR_MANAGER (manager_kms);
+  unsigned int i;
+  gboolean connected_crtc_found;
+
+  if (manager->power_save_mode != META_POWER_SAVE_ON)
+    return FALSE;
+
+  connected_crtc_found = FALSE;
+  for (i = 0; i < manager->n_outputs; i++)
+    {
+      MetaOutput *output = &manager->outputs[i];
+
+      if (output->crtc == crtc)
+        {
+          connected_crtc_found = TRUE;
+          break;
+        }
+    }
+
+  if (!connected_crtc_found)
+    return FALSE;
+
+  return TRUE;
+}
+
+gboolean
+meta_monitor_manager_kms_flip_crtc (MetaMonitorManagerKms *manager_kms,
+                                    MetaCRTC              *crtc,
+                                    int                    x,
+                                    int                    y,
+                                    uint32_t               fb_id,
+                                    GClosure              *flip_closure)
+{
+  MetaMonitorManager *manager = META_MONITOR_MANAGER (manager_kms);
+  uint32_t *connectors;
+  unsigned int n_connectors;
+  int ret = -1;
+
+  g_assert (manager->power_save_mode == META_POWER_SAVE_ON);
+
+  get_crtc_connectors (manager, crtc, &connectors, &n_connectors);
+  g_assert (n_connectors > 0);
+
+  if (!manager_kms->page_flips_not_supported)
+    {
+      ret = drmModePageFlip (manager_kms->fd,
+                             crtc->crtc_id,
+                             fb_id,
+                             DRM_MODE_PAGE_FLIP_EVENT,
+                             flip_closure);
+      if (ret != 0 && ret != -EACCES)
+        {
+          g_warning ("Failed to flip: %s", strerror (-ret));
+          manager_kms->page_flips_not_supported = TRUE;
+        }
+    }
+
+  if (manager_kms->page_flips_not_supported)
+    meta_monitor_manager_kms_apply_crtc_mode (manager_kms,
+                                              crtc,
+                                              x, y,
+                                              fb_id);
+
+  if (ret != 0)
+    return FALSE;
+
+  g_closure_ref (flip_closure);
+
+  return TRUE;
+}
+
+static void
+page_flip_handler (int fd,
+                   unsigned int frame,
+                   unsigned int sec,
+                   unsigned int usec,
+                   void *data)
+{
+  GClosure *flip_closure = data;
+
+  invoke_flip_closure (flip_closure);
+}
+
+void
+meta_monitor_manager_kms_wait_for_flip (MetaMonitorManagerKms *manager_kms)
+{
+  drmEventContext evctx;
+
+  if (manager_kms->page_flips_not_supported)
+    return;
+
+  memset (&evctx, 0, sizeof evctx);
+  evctx.version = DRM_EVENT_CONTEXT_VERSION;
+  evctx.page_flip_handler = page_flip_handler;
+  drmHandleEvent (manager_kms->fd, &evctx);
+}
+
 static void
 meta_monitor_manager_kms_dispose (GObject *object)
 {
@@ -1335,6 +1476,7 @@ meta_monitor_manager_kms_finalize (GObject *object)
   MetaMonitorManagerKms *manager_kms = META_MONITOR_MANAGER_KMS (object);
 
   free_resources (manager_kms);
+  g_source_destroy ((GSource *) manager_kms->source);
 
   G_OBJECT_CLASS (meta_monitor_manager_kms_parent_class)->finalize (object);
 }
