@@ -37,4 +37,24 @@ typedef struct _MetaMonitorManagerKms         MetaMonitorManagerKms;
 
 GType meta_monitor_manager_kms_get_type (void);
 
+typedef void (*MetaKmsFlipCallback) (void *user_data);
+
+void meta_monitor_manager_kms_apply_crtc_mode (MetaMonitorManagerKms *manager_kms,
+                                               MetaCRTC              *crtc,
+                                               int                    x,
+                                               int                    y,
+                                               uint32_t               fb_id);
+
+gboolean meta_monitor_manager_kms_is_crtc_active (MetaMonitorManagerKms *manager_kms,
+                                                  MetaCRTC              *crtc);
+
+gboolean meta_monitor_manager_kms_flip_crtc (MetaMonitorManagerKms *manager_kms,
+                                             MetaCRTC              *crtc,
+                                             int                    x,
+                                             int                    y,
+                                             uint32_t               fb_id,
+                                             GClosure              *flip_closure);
+
+void meta_monitor_manager_kms_wait_for_flip (MetaMonitorManagerKms *manager_kms);
+
 #endif /* META_MONITOR_MANAGER_KMS_H */
