@@ -176,9 +176,6 @@ struct _MetaWindow
   /* Whether the window is marked as urgent */
   guint urgent : 1;
 
-  /* Whether we have to fullscreen after placement */
-  guint fullscreen_after_placement : 1;
-
   /* Area to cover when in fullscreen mode.  If _NET_WM_FULLSCREEN_MONITORS has
    * been overridden (via a client message), the window will cover the union of
    * these monitors.  If not, this is the single monitor which the window's
@@ -419,8 +416,10 @@ struct _MetaWindow
    *
    * For X11 windows, this matches XGetGeometry of the toplevel.
    *
-   * For Wayland windows, this matches the buffer size and where
-   * the surface actor is positioned. */
+   * For Wayland windows, the position matches the position of the
+   * surface associated with shell surface (wl_shell_surface, xdg_surface
+   * etc). The size matches the size surface size as displayed in the stage.
+   */
   MetaRectangle buffer_rect;
 
   /* Cached net_wm_icon_geometry */
