@@ -35,6 +35,13 @@
 #include "wayland/meta-window-wayland.h"
 #include "xdg-shell-unstable-v5-server-protocol.h"
 
+/*
+ * Define GNOME additional states to xdg-shell
+ * The current reserved range for GNOME is 0x1000 - 0x1FFF
+ */
+
+#define XDG_SURFACE_STATE_GNOME_TILED 0x1000
+
 struct _MetaWaylandXdgSurface
 {
   MetaWaylandSurfaceRoleShellSurface parent;
@@ -136,6 +143,9 @@ xdg_surface_set_title (struct wl_client   *client,
 {
   MetaWaylandSurface *surface = surface_from_xdg_surface_resource (resource);
 
+  if (!g_utf8_validate (title, -1, NULL))
+    title = "";
+
   meta_window_set_title (surface->window, title);
 }
 
@@ -146,6 +156,9 @@ xdg_surface_set_app_id (struct wl_client   *client,
 {
   MetaWaylandSurface *surface = surface_from_xdg_surface_resource (resource);
 
+  if (!g_utf8_validate (app_id, -1, NULL))
+    app_id = "";
+
   meta_window_set_wm_class (surface->window, app_id, app_id);
 }
 
@@ -393,6 +406,13 @@ fill_states (struct wl_array *states, MetaWindow *window)
       s = wl_array_add (states, sizeof *s);
       *s = XDG_SURFACE_STATE_ACTIVATED;
     }
+  /* GNOME extension to xdg-shell states */
+  if (window->tile_mode == META_TILE_LEFT ||
+      window->tile_mode == META_TILE_RIGHT)
+    {
+      s = wl_array_add (states, sizeof *s);
+      *s = XDG_SURFACE_STATE_GNOME_TILED;
+    }
 }
 
 static void
